package laboratoire4.pawns;

import laboratoire4.IPawn;
import laboratoire4.Player;

public abstract class Pawn implements IPawn {
    protected final Player player;
    protected int row;
    protected int col;

    public Pawn(Player player, int row, int col) {
        this.player = player;
        this.row = row;
        this.col = col;
    }

    public void move(PawnMovement movement) {
        setRow(row + player.getDirection());
        setCol(col + movement.getMove());
    }

    public String getPosition() {
        char colStr = (char) (col + 65);
        return String.format("%s%d", colStr, row + 1);
    }

    public Player getPlayer() {
        return player;
    }

    public int getCol() {
        return col;
    }

    public int getRow() {
        return row;
    }

    public void setCol(int col) {
        this.col = col;
    }

    public void setRow(int row) {
        this.row = row;
    }

    public int getDirection() {
        return player.getDirection();
    }

    public boolean isMoveValid(IPawn[][] board, PawnMovement movement) {
        return isMoveValid(board, movement, false);
    }

    public boolean isMoveValid(IPawn[][] board, PawnMovement movement, boolean ignorePlayers) {
        return isMoveValid(board, movement, row, col, ignorePlayers);
    }

    public boolean isMoveValid(IPawn[][] board, PawnMovement movement, int fromRow, int fromCol) {
        return isMoveValid(board, movement, fromRow, fromCol, false);
    }

    @Override
    public boolean isMoveValid(IPawn[][] board, PawnMovement movement, int fromRow, int fromCol, boolean ignorePlayers) {
        int nextRow = fromRow + getDirection();
        if (nextRow < 0 || nextRow >= board.length) {
            return false;
        }

        int nextCol = fromCol + movement.getMove();
        if (nextCol < 0 || nextCol >= board.length) {
            return false;
        }

        return isMoveReallyValid(board, movement, fromRow, fromCol, ignorePlayers);
    }

    protected abstract boolean isMoveReallyValid(IPawn[][] board, PawnMovement movement, int fromRow, int fromCol, boolean ignorePlayers);
}
