package laboratoire4.strategies;

import laboratoire4.Action;
import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.game.SimulatedGame;
import laboratoire4.pawns.PawnMovement;
import laboratoire4.pawns.PawnUtils;
import laboratoire4.pawns.SimulatedPawn;

import java.util.Collection;

public abstract class MiniMaxStrategy implements Strategy {
    private static final int MAX_DEPTH = 6;

    protected SimulatedGame game;

    @Override
    public EvaluationResult getNextMove(Game game) {
        return miniMax(game);
    }

    private EvaluationResult miniMax(Game game) {
        this.game = new SimulatedGame(game.getBoard(), game.getPlayer());

        EvaluationResult maxResult = null;
        int maxScore = Integer.MIN_VALUE;

        for (Action<IPawn> action : getActions(true)) {
            int score = min(0, Integer.MIN_VALUE, Integer.MAX_VALUE);

            if (maxResult == null || score > maxScore) {
                IPawn pawn = action.getPawn();
                maxResult = new EvaluationResult(pawn.getRow(), pawn.getCol(), action.getMovement());
            }
        }

        return maxResult;
    }

    private int max(int depth, int alpha, int beta) {
        if (depth >= MAX_DEPTH) {
            return evaluate();
        }

        int maxScore = Integer.MIN_VALUE;

        for (Action<IPawn> action : getActions(true)) {
            IPawn pawn = action.getPawn();
            PawnMovement movement = action.getMovement();

            game.move(pawn, movement);
            int score = min(depth + 1, Math.max(alpha, maxScore), beta);
            game.revertMove();

            if (score > maxScore) {
                maxScore = score;
            }

            if (maxScore >= beta) {
                return maxScore;
            }
        }

        return maxScore;
    }

    private int min(int depth, int alpha, int beta) {
        if (depth >= MAX_DEPTH) {
            return evaluate();
        }

        int minScore = Integer.MAX_VALUE;

        for (Action<IPawn> action : getActions(false)) {
            IPawn pawn = action.getPawn();
            PawnMovement movement = action.getMovement();

            game.move(pawn, movement);
            int score = max(depth + 1, alpha, Math.min(beta, minScore)) * -1;
            game.revertMove();

            if (score < minScore) {
                minScore = score;
            }

            if (minScore <= alpha) {
                return minScore;
            }
        }

        return minScore;
    }

    protected Collection<SimulatedPawn> getMaxPawns() {
        return GameUtils.getMaxPawns(game.getBoard(), game.getPlayer());
    }

    protected Collection<SimulatedPawn> getMinPawns() {
        return GameUtils.getMinPawns(game.getBoard(), game.getPlayer());
    }

    protected Collection<Action<IPawn>> getActions(boolean max) {
        return PawnUtils.getActions(game, max);
    }

    private int evaluate() {
        int score = evaluateSimulation();

        // Logique générale

        return score;
    }

    protected abstract int evaluateSimulation();
}
